@@ -5,14 +5,16 @@ import six
 import jmespath
 from jmespath.exceptions import ParseError
 import logging
+from logging.handlers import RotatingFileHandler
+from .exceptions import IncompleteAccountInfoError
 
 LOG_CLIENT_METHOD_BLACK_LIST = (r'_.+', r'\w+acl', 'set_source', 'delete_shard', 'heart_beat',
-                                'set_user_agent', 'get_unicode', 'list_logstores'
+                                'set_user_agent', 'get_unicode', 'list_logstores', 'put_log_raw'
                                 )
 
 LOG_CREDS_FILENAME = "%s/.aliyunlogcli" % os.path.expanduser('~')
 DEFAULT_DEBUG_LOG_FILE_PATH = "%s/aliyunlogcli.log" % os.path.expanduser('~')
-DEFAULT_DEBUG_LOG_FORMAT = "%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(funcName)s %(message)s"
+DEFAULT_DEBUG_LOG_FORMAT = "%(asctime)s %(levelname)s %(threadName)s:%(filename)s:%(lineno)d %(funcName)s %(message)s"
 
 LOG_CONFIG_SECTION = "main"
 GLOBAL_OPTION_SECTION = "__option__"
@@ -29,17 +31,17 @@ SystemConfig = namedtuple('SystemConfig', "access_id access_key endpoint jmes_fi
 API_GROUP = [('project$', 'Project'), 'logstore', ('index|topics', "Index"),
              ('logtail_config', "Logtail Config"), ('machine', "Machine Group"), 'shard',
              'cursor', ('log|histogram', "Logs"), ('consumer|check_point', "Consumer Group"), 'shipper',
-             'dashboard', 'savedsearch', 'alert']
+             'dashboard', 'savedsearch', 'alert', ('external_store', "External Store")]
 
 GLOBAL_OPTIONS_STR = """
 Global Options:
-[--access-id=<value>]		: use this access id in this command
-[--access-key=<value>]		: use this access key in this command
-[--region-endpoint=<value>]	: use this endpoint in this command
-[--client-name=<value>]		: use this client name in configured accounts
-[--jmes-filter=<value>]		: filter results using JMES syntax
-[--format-output=json]		: print formatted json results or else print in one line
-[--decode-output=<value>]	: encoding list to decode response, comma separated like "utf8,lartin1,gbk", default is "utf8". 
+[--access-id=<value>]		        : use this access id in this command
+[--access-key=<value>]		        : use this access key in this command
+[--region-endpoint=<value>]	        : use this endpoint in this command
+[--client-name=<value>]		        : use this client name in configured accounts
+[--jmes-filter=<value>]		        : filter results using JMES syntax
+[--format-output=json,no_escape]    : print formatted json results or else print in one line; if escape non-ANSI or not with `no_escape`. like: "json", "json,no_escape", "no_escape"
+[--decode-output=<value>]	        : encoding list to decode response, comma separated like "utf8,lartin1,gbk", default is "utf8". 
 
 Refer to http://aliyun-log-cli.readthedocs.io/ for more info.
 """
@@ -49,22 +51,22 @@ Usage:
 
 1. aliyunlog log <subcommand> [parameters | global options]
 2. aliyunlog configure <access_id> <access-key> <endpoint> [<client-name>]
-3. aliyunlog configure [--format-output=json] [--default-client=<client_name>] [--decode-output=utf8,latin1]
+3. aliyunlog configure [--format-output=json,no_escape] [--default-client=<client_name>] [--decode-output=utf8,latin1]
 4. aliyunlog [--help | --version]
 
 
 Examples:
 
 1. aliyunlog configure AKID123 AKKEY123 cn-hangzhou.log.aliyuncs.com
-2. aliyunlog configure --format-output=json --default-client=beijing
+2. aliyunlog configure --format-output=json,no_escape --default-client=beijing
 3. aliyunlog log create_project --project_name="test"
 
 Subcommand:
 {grouped_api}
 """ + GLOBAL_OPTIONS_STR
 
-MORE_DOCOPT_CMD = """aliyun configure <secure_id> <secure_key> <endpoint> [<client_name>]
-aliyun configure [--format-output=json] [--default-client=<client_name>] [--decode-output=utf8,latin1]
+MORE_DOCOPT_CMD = """aliyunlog configure <secure_id> <secure_key> <endpoint> [<client_name>]
+aliyunlog configure [--format-output=json,no_escape] [--default-client=<client_name>] [--decode-output=utf8,latin1]
 """
 
 DEBUG_SECTION_NAME = "__logging__"
@@ -136,7 +138,8 @@ def load_config(system_options):
     format_output = system_options.get('format-output', format_output)
     decode_output = system_options.get('decode-output', decode_output)
 
-    assert access_id and access_key and endpoint, ValueError("Access id/key or endpoint is empty!")
+    if not (access_id and access_key and endpoint):
+        raise IncompleteAccountInfoError("Access id/key or endpoint is empty!")
 
     # load jmes filter from cmd
     jmes_filter = system_options.get('jmes-filter', '')
@@ -164,29 +167,49 @@ __LOGGING_LEVEL_MAP = {
 }
 
 
-def load_debug_from_config_file():
+def config_logging_from_config_file():
     # load debug config from file
     config = configparser.RawConfigParser()
     config.read(LOG_CREDS_FILENAME)
 
-    opt = {"filename": DEFAULT_DEBUG_LOG_FILE_PATH, "level": logging.WARN, "format": DEFAULT_DEBUG_LOG_FORMAT}
+    filename = DEFAULT_DEBUG_LOG_FILE_PATH
+    level = "warn"          # use string first
+    fmt = DEFAULT_DEBUG_LOG_FORMAT
+    datefmt = None
+    filebytes = 100 * 1024 * 1024
+    backupcount = 5
+
     client_name = DEBUG_SECTION_NAME
     if config.has_section(client_name):
-        filename = _get_section_option(config, client_name, 'filename', None)
-        level = _get_section_option(config, client_name, 'level', None)
-        filemode = _get_section_option(config, client_name, 'filemode', None)
-        fmt = _get_section_option(config, client_name, 'format', None)
-        datefmt = _get_section_option(config, client_name, 'datefmt', None)
+        filename = _get_section_option(config, client_name, 'filename', filename)
+        level = _get_section_option(config, client_name, 'level', level)
+        fmt = _get_section_option(config, client_name, 'format', fmt)
+        datefmt = _get_section_option(config, client_name, 'datefmt', datefmt)
+        filebytes = int(_get_section_option(config, client_name, 'filebytes', filebytes))
+        backupcount = int(_get_section_option(config, client_name, 'backupcount', backupcount))
 
-        if filename is not None:
-            opt['filename'] = filename
-        if filemode is not None:
-            opt['filemode'] = filemode
-        if fmt is not None:
-            opt['format'] = fmt
-        if datefmt is not None:
-            opt['datefmt'] = datefmt
+    root = logging.getLogger()
+    handler = RotatingFileHandler(filename, maxBytes=filebytes, backupCount=backupcount)
+    root.setLevel(__LOGGING_LEVEL_MAP.get(level.lower().strip(), logging.WARN))
+    root.addHandler(handler)
+    handler.setFormatter(logging.Formatter(fmt=fmt, datefmt=datefmt))
 
-        opt['level'] = __LOGGING_LEVEL_MAP.get(level.lower().strip(), logging.WARN)
 
-    return opt
+class monkey_patch(object):
+    def __init__(self, src_obj, src_prop, dst=None):
+        self.src_obj = src_obj
+        self.src_prop = src_prop
+        self.activated = False
+
+        if hasattr(self.src_obj, self.src_prop):
+            self.origin = getattr(self.src_obj, self.src_prop)
+            setattr(self.src_obj, self.src_prop, dst)
+            self.activated = True
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.activated:
+            setattr(self.src_obj, self.src_prop, self.origin)
+
